# -*- coding: utf-8 -*-
from setuphelpers import *
import platform

uninstallkey = []

# Declaring specific app values (TO CHANGE)
bin_name_deb_string = "code-%s.deb"
bin_name_rpm_string = "code-%s.rpm"
package_name = "code"


def install():
    # Declaring specific app values
    package_version = control["version"].split("-", 1)[0]

    # Installing the package
    if platform.linux_distribution()[0].lower() in ["debian", "ubuntu"]:
        print("Installing deb package")
        dpkg_install(bin_name_deb_string % package_version)

    if platform.linux_distribution()[0].lower() in [
        "centos linux",
        "centos",
        "suse",
        "fedora",
        "redhat",
    ]:
        print("Installing yum package")
        yum_install(bin_name_rpm_string % package_version)


def uninstall():
    # Declaring specific app values
    package_version = control["version"].split("-", 1)[0]

    # Uninstalling the package
    if platform.linux_distribution()[0].lower() in ["debian", "ubuntu"]:
        print("Uninstalling deb package")
        apt_remove(package_name)

    if platform.linux_distribution()[0].lower() in [
        "centos linux",
        "centos",
        "suse",
        "fedora",
        "redhat",
    ]:
        print("Uninstalling yum package")
        yum_remove(package_name)


def audit():
    print("Disabling : telemetry, check for update, updating, update note tab")
    import json

    # https://developer.hyvor.com/vscode-editing-settings-json

    # /root/.config/Code/User/settings.json
    # /home/user/.config/Code/User/settings.json

    settings_json_content = {
        "telemetry.enableTelemetry": False,
        "telemetry.enableCrashReporter": False,
        "update.mode": "none",
        "update.enableWindowsBackgroundUpdates": False,
        "update.showReleaseNotes": False,
    }

    conf_dir_path = makepath(".config", "Code", "User")
    conf_file_path = makepath(conf_dir_path, "settings.json")
    root_conf_dir_path = makepath("/", "root", conf_dir_path)
    root_conf_file_path = makepath(root_conf_dir_path, "settings.json")

    print("Configuring : " + root_conf_file_path)
    if not isfile(root_conf_file_path):
        with open(root_conf_file_path, "w+") as outfile:
            json.dump(settings_json_content, outfile, indent=4)
    else:
        with open(root_conf_file_path, "r+") as f:
            settings_json_outfile = json.load(f)
            settings_json_outfile.update(settings_json_content)
            f.seek(0, 0)
            json.dump(settings_json_outfile, f, indent=4)

    for users in glob.glob(makepath("/", "home", "*")):
        user_conf_dir_path = makepath(users, conf_dir_path)
        user_conf_file_path = makepath(users, conf_file_path)
        print("Configuring : " + user_conf_file_path)
        # if not isdir(user_conf_dir_path):
        #    mkdirs(user_conf_dir_path)

        if not isfile(user_conf_file_path):
            with open(user_conf_file_path, "w+") as outfile:
                json.dump(settings_json_content, outfile, indent=4)
        else:
            with open(user_conf_file_path, "r+") as f:
                settings_json_outfile = json.load(f)
                settings_json_outfile.update(settings_json_content)
                f.seek(0, 0)
                json.dump(settings_json_outfile, f, indent=4)
    return "OK"


def update_package():
    print("Download/Update package content from upstream binary sources")
    import requests

    # Update transition from BeautifulSoup 3 to 4
    try:
        import bs4 as BeautifulSoup
    except:
        import BeautifulSoup

    # Getting proxy informations from WAPT settings
    proxy = {}
    if isfile(makepath(application_data(), "waptconsole", "waptconsole.ini")):
        proxywapt = inifile_readstring(
            makepath(user_local_appdata(), "waptconsole", "waptconsole.ini"),
            "global",
            "http_proxy",
        )
        if proxywapt:
            proxy = {"http": proxywapt, "https": proxywapt}

    # Specific app values
    app_name = control.name
    url = "https://github.com/microsoft/vscode/tags"

    # Getting latest version from official website
    page = requests.get(url).text
    bs = BeautifulSoup.BeautifulSoup(page)
    # bs = BeautifulSoup.BeautifulSoup(page,features="html.parser") # For bs4 only
    bs_raw_string = bs.find("h4", {"class": "flex-auto min-width-0 pr-2 pb-1 commit-title"})
    version = (bs_raw_string.findNext())["href"].split("/")[-1]

    url_dl_deb = "https://go.microsoft.com/fwlink/?LinkID=760867"
    url_dl_yum = "https://go.microsoft.com/fwlink/?LinkID=760868"

    latest_bin_deb = bin_name_deb_string % version
    latest_bin_yum = bin_name_rpm_string % version

    print("Latest " + app_name + " version is: " + version)
    print("Download url is: " + url_dl_deb + " and " + url_dl_yum)

    # Downloading latest binaries
    if not isfile(latest_bin_deb):
        print("Downloading: " + latest_bin_deb)
        wget(url_dl_deb, latest_bin_deb, proxies=proxy)

    if not isfile(latest_bin_yum):
        print("Downloading: " + latest_bin_yum)
        wget(url_dl_yum, latest_bin_yum, proxies=proxy)

        # Change version of the package
        from waptpackage import PackageEntry

        print("Writing " + version + "-0 in WAPT\\control")
        pe = PackageEntry()
        pe.load_control_from_wapt(os.getcwd())

        pe.version = version + "-0"
        pe.save_control_to_wapt(os.getcwd())
        print("Update package done. You can now build-upload your package")
    else:
        print("This package is already up-to-date")

    # Deleting outdated binaries
    for bin_in_dir in glob.glob("*.deb") and glob.glob("*.rpm"):
        if bin_in_dir != latest_bin_yum and bin_in_dir != latest_bin_deb:
            print("Outdated binary: " + bin_in_dir + " Deleted")
            remove_file(bin_in_dir)


if __name__ == "__main__":
    update_package()
