#!waptpython
# -*- coding: utf-8 -*-
import waptpackage
import json
import subprocess
import deepl
import pathlib
import unicodedata
import os
import sys

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))
from setupdevhelpers import *
from dev_utils import *

sys.path.insert(0, sys.executable.rsplit(os.path.sep, 1)[0].replace("/bin", "").replace("\\Scripts", ""))
from setuphelpers import *

from typing import List, Dict
from PIL import Image

try:
    import waptguihelper
except:
    pass
if not get_os_name() == "Darwin":
    from icoextract import IconExtractor, logger, __version__


def makepath_function(file):
    def makepath_to_file(pkg_dir):
        return makepath(pkg_dir, file)

    return makepath_to_file


# def makepath_function(file):
#     return lambda pkg_dir: makepath(pkg_dir, file)


# makepath_function("setup.py")(pkg_dir)
# setup_py_makepath(pkg_dir)


def setup_py_makepath(pkg_dir):
    return makepath(pkg_dir, "setup.py")


def update_package_py_makepath(pkg_dir):
    return makepath(pkg_dir, "update_package.py")


def control_makepath(pkg_dir):
    if "WAPT%scontrol" % os.sep in pkg_dir:
        return pkg_dir
    else:
        return makepath(pkg_dir, "WAPT", "control")


def icon_makepath(pkg_dir):
    return makepath(pkg_dir, "WAPT", "icon.png")


def changelog_txt_makepath(pkg_dir):
    return makepath(pkg_dir, "WAPT", "changelog.txt")


def icon_png_makepath(pkg_dir):
    return makepath(pkg_dir, "WAPT", "icon.png")


def control_file_unified(
    dst,
    src,
    answer_os_interaction,
    keys_to_clear_list,
    operating_system=None,
    arch=None,
    lang=None,
):
    control_src_dict = get_control_dict(src)
    if isfile(makepath(dst, "WAPT", "control")):
        control_dst_dict = get_control_dict(dst)
    else:
        control_dst_dict = control_src_dict
    control_uni_dict = {}

    for key in control_src_dict.keys():
        if control_src_dict[key]:
            control_uni_dict[key] = control_src_dict[key]
        else:
            control_uni_dict[key] = control_dst_dict[key]

    if control_uni_dict["version"] == "0":
        control_uni_dict["version"] = control_dst_dict["version"]
    if not "-" in control_uni_dict["version"]:
        control_uni_dict["version"] = control_uni_dict["version"] + "-0"
    if not arch:
        arch = control_dst_dict["architecture"]
    if not lang:
        lang = "all"

    control_uni_dict["architecture"] = arch
    control_uni_dict["locale"] = lang

    if operating_system is not None:
        if len(control_dst_dict["target_os"].split(",")) > 1 or control_dst_dict["target_os"].startswith(operating_system):
            control_uni_dict["target_os"] = control_dst_dict["target_os"]
        else:
            if operating_system.startswith(control_dst_dict["target_os"]):
                if not control_dst_dict["min_os_version"] or control_dst_dict["max_os_version"]:
                    control_uni_dict["target_os"] = operating_system
            else:
                control_uni_dict["target_os"] = operating_system
    control_uni_dict["target_os"] = control_uni_dict["target_os"].lower()

    # limit changes if different os_dir
    if src.split(os.sep)[0] != dst.split(os.sep)[0]:
        control_uni_dict["min_os_version"] = control_dst_dict["min_os_version"]
        control_uni_dict["max_os_version"] = control_dst_dict["max_os_version"]
        control_uni_dict["depends"] = control_dst_dict["depends"]
        control_uni_dict["conflicts"] = control_dst_dict["conflicts"]
    # force changes if same os_dir
    else:
        control_uni_dict["installed_size"] = control_src_dict["installed_size"]
        control_uni_dict["depends"] = control_src_dict["depends"]
        control_uni_dict["conflicts"] = control_src_dict["conflicts"]
    # force some changes if os is all
    if answer_os_interaction == "all":
        control_uni_dict["depends"] = control_src_dict["depends"]
        control_uni_dict["conflicts"] = control_src_dict["conflicts"]
        control_uni_dict["installed_size"] = control_src_dict["installed_size"]

    if control_uni_dict["maturity"] == "":
        control_uni_dict["maturity"] = "PROD"

    for key in keys_to_clear_list:
        del control_uni_dict[key]

    # # Increment if any translation
    # if descriptions_updated:
    #     new_pkg_version = int(control_dst_dict["version"].split("-", 1)[-1]) + 1
    #     control_uni_dict["version"] = "%s-%s" % (control_dst_dict["version"].split("-", 1)[0], new_pkg_version)

    new_pkg_entry = waptpackage.PackageEntry().load_control_from_dict(control_uni_dict)
    new_pkg_entry.save_control_to_wapt(dst)


def overwrite_older_files(package_dirs, makepath_function):
    overwritten_files = []
    newer_file = get_newer_file_from_dirs(package_dirs, makepath_function)
    newer_file_sha256 = get_file_sha256(newer_file)

    for package_dir in package_dirs:
        file_path = update_package_py_makepath(package_dir)
        if file_path == newer_file:
            continue
        file_sha256 = get_file_sha256(file_path)
        if file_sha256 != newer_file_sha256:
            filecopyto(newer_file, file_path)
            print(f'Overwritting: "{file_path}" with: "{newer_file}"')


def get_newer_file_from_dirs(package_dirs, makepath_function):
    newest_file = None
    newest_file_date = 0

    for package_dir in package_dirs:
        file_path = makepath_function(package_dir)
        if isfile(file_path):
            file_date = os.path.getmtime(file_path)
            if file_date > newest_file_date:
                newest_file_date = file_date
                newest_file = file_path
    return newest_file


def get_newer_file(pkg_dir_list, makepath_function, os_based=False, pkg_os_dir=None):
    """
    Returns the newest file in pkg_dir_list based on modification time.
    If os_based is True, only returns files with the same first digit in the parent directory name as pkg_os_dir.
    """
    if os_based and pkg_os_dir is None:
        raise ValueError("Please provide pkg_os_dir")

    newest_file_date = 0
    newest_file = None

    if os.access(makepath_function("template"), os.R_OK):
        pkg_dir_list.append("template")

    for pkg_dir in pkg_dir_list:
        if pkg_dir == "template":
            continue
        elif os_based:
            parent_dir = os.path.basename(os.path.dirname(makepath_function(pkg_dir)))
            if not parent_dir[0] == pkg_os_dir[0]:
                continue
        try:
            file_date = os.path.getmtime(makepath_function(pkg_dir))
        except OSError:
            continue
        if file_date > newest_file_date:
            newest_file_date = file_date
            newest_file = makepath_function(pkg_dir)

    return newest_file


def get_deepl_auth_key(force_change=False):
    json_file_path = makepath(user_appdata, "waptdev", "config.json")
    to_save = False
    if os.path.exists(json_file_path):
        json_data = json_load_file(json_file_path)
        if not json_data.get("deepl_auth_key"):
            to_save = True
        deepl_auth_key = json_data.get("deepl_auth_key") if not json_data.get("deepl_api") else json_data.get("deepl_api")

    if force_change or not deepl_auth_key or to_save:
        prompt = "Please provide your DeepL API auth_key" if not force_change else "Please provide your new DeepL API auth_key"
        deepl_auth_key = ask_input(
            "DeepL API auth_key", prompt, "deepl_auth_key" if not force_change and not deepl_auth_key else deepl_auth_key, raise_error=True
        )
        to_save = True

    if to_save:
        save_deepl_auth_key(deepl_auth_key)

    return deepl_auth_key


def save_deepl_auth_key(deepl_auth_key):
    json_file_path = makepath(user_appdata, "waptdev", "config.json")
    if os.path.exists(json_file_path):
        json_data = json_load_file(json_file_path)
        json_data = json_load_file(json_file_path)
    else:
        json_data = {}
    json_data.update({"deepl_auth_key": deepl_auth_key})
    if json_data.get("deepl_api"):
        del json_data["deepl_api"]
    mkdirs(os.path.dirname(json_file_path))
    with open(json_file_path, "w") as json_file:
        json.dump(json_data, json_file)
    print("API key saved to JSON file")


def set_translator_deepl(deepl_auth_key, proxies=None):
    """https://github.com/DeepLcom/deepl-python#readme
    default:
        deepl.Translator(deepl_auth_key, server_url=None, proxy=None, send_platform_info=True, verify_ssl=None)
    """
    return deepl.Translator(deepl_auth_key, server_url=None, proxy=proxies, send_platform_info=False, verify_ssl=None)


def detect_source_language(translator, text):
    try:
        result = translator.translate_text(text, target_lang="EN-US")
    except Exception as e:
        print(e)
        if "Authorization failure, check auth_key" in str(e):
            get_deepl_auth_key(force_change=True)
            error("Please rerun")

    return result.detected_source_lang


def translate_text(translator, text, source_lang=None, target_lang=None, formality=None, glossary=None):
    """
    Translate the given text using the provided translator object.

    Args:
        translator (Translator): An instance of the Translator class for performing translations.
        text (str): The input text to be translated.
        source_lang (str, optional): Specifies the source language code. If not provided, auto-detection will be attempted.
        target_lang (str): Specifies the target language code for the translation.
        formality (str, optional): Controls the formality of translations, available only for certain languages.
            - 'default': Default.
            - 'more': Translations will be more formal and polite.
            - 'less': Translations will lean toward informal language.
            - 'prefer_more': for a more formal language if available, otherwise fallback to default formality
            - 'prefer_less': for a more informal language if available, otherwise fallback to default formality
        glossary (str or GlossaryInfo, optional): Specifies a glossary for translation, either as a glossary ID string
            or a GlossaryInfo object as returned by get_glossary().

    Returns:
        str: Translated text in the target language.
    """
    try:
        result = translator.translate_text(text, source_lang=source_lang, target_lang=target_lang, formality=formality, glossary=glossary)
    except Exception as e:
        print(e)
        if "Authorization failure, check auth_key" in str(e):
            get_deepl_auth_key(force_change=True)
            error("Please rerun")

    return result.text


def translate_control_descriptions(translator, control, formality=None, glossary=None):
    if not "PackageEntry" in str(type(control)):
        pkg_dir = convert_control_path_to_dir(control)
        control = PackageEntry().load_control_from_wapt(pkg_dir)
        control_dict = control.as_dict()
    old_control_dict = control_dict
    descriptions_dict = {
        "description": "",
        "description_fr": "",
        "description_pl": "",
        "description_de": "",
        "description_es": "",
        "description_pt": "",
        "description_it": "",
        "description_nl": "",
        "description_ru": "",
    }
    descriptions_updated = False
    filled_descriptions = 0

    for key in control_dict.keys():
        if "description" in key:
            if control_dict[key].strip():
                descriptions_dict[key] = control_dict[key]
                filled_descriptions += 1

    # Translate if needed
    if len(descriptions_dict) <= filled_descriptions:
        print("Descriptions already translated, to force translation you must clear additionnal descriptions of any control file.")
    else:
        if not control_dict["description"]:
            error("Please fill control.description to translate_control_descriptions.")
        else:
            # Detect control.description language to fill correct field
            source_text = control_dict["description"]
            source_lang = detect_source_language(translator, source_text)
            control_lang = source_lang.lower().split("-")[0]
            if not "EN" in source_lang:
                print(f"Detected source language: {source_lang}")
                control_dict[f"description_{control_lang}"] = source_text
                control_dict["description"] = translate_text(translator, source_text, source_lang=source_lang, target_lang="EN-US")
                descriptions_updated = True

            print(f"Translating missing descriptions with source [{source_lang}]: {source_text}\n")
            for key in control_dict.keys():
                if "description" in key and not control_dict[key].strip():
                    target_lang = key.rsplit("_", 1)[-1]
                    if target_lang == "pt":
                        target_lang = "PT-PT"

                    translated_description = translate_text(translator, source_text, source_lang=source_lang, target_lang=target_lang)

                    # update description and print message and retire dot in the end of the string
                    control_dict[key] = translated_description.rsplit(".", 1)[0] if translated_description.endswith(".") else translated_description
                    print(f"{key}: {translated_description}\n")
            descriptions_updated = True

    # Retire dot in the end of the string
    control_dict["description"] = (
        control_dict["description"].rsplit(".", 1)[0] if control_dict["description"].endswith(".") else control_dict["description"]
    )
    control.load_control_from_dict(control_dict)
    control.save_control_to_wapt()

    return descriptions_updated


def get_file_sha256(fp):
    if fp is not None:
        if isfile(fp):
            return get_sha256(fp)
    return ""


# def get_size(start_path="."):
#     total_size = 0
#     for dirpath, dirnames, filenames in os.walk(start_path):
#         for f in filenames:
#             fp = os.path.join(dirpath, f)
#             # skip if it is symbolic link
#             if not os.path.islink(fp):
#                 total_size += os.path.getsize(fp)
#     return total_size


def save_maintainer_name(maintainer_name):
    appdata_path = os.getenv("APPDATA")
    json_file_path = os.path.join(appdata_path, "waptdev", "config.json")
    if os.path.exists(json_file_path):
        with open(json_file_path, "r") as json_file:
            json_data = json.load(json_file)
        json_data["maintainer_name"] = maintainer_name
        with open(json_file_path, "w") as json_file:
            json.dump(json_data, json_file)
        print("Maintainer name saved to JSON file")
    else:
        os.makedirs(os.path.dirname(json_file_path), exist_ok=True)
        json_data = {"maintainer_name": maintainer_name}
        with open(json_file_path, "w") as json_file:
            json.dump(json_data, json_file)
        print("Maintainer name saved to JSON file")


def get_maintainer_name():
    appdata_path = os.getenv("APPDATA")
    json_file_path = os.path.join(appdata_path, "waptdev", "config.json")
    if os.path.exists(json_file_path):
        with open(json_file_path, "r") as json_file:
            json_data = json.load(json_file)
            if "maintainer_name" in json_data:
                maintainer_name = json_data["maintainer_name"]
                return maintainer_name
            else:
                print("JSON file is missing maintainer_name property")
                return None
    else:
        print("JSON file not found in appdata")
        return None


def extract_icon(icon_path, output_path):
    """Windows only, uses tis-extracticon"""
    extracticon_path = makepath(programfiles32, "extracticon", "extracticon.exe")
    try:
        run([extracticon_path, icon_path, output_path])
    except:
        error(f"ERROR: {extracticon_path} failed.\nTry to reinstall the package tis-extracticon")


def get_icoextract_path():
    if get_os_name() == "Windows":
        return os.sep.join([sys.executable.rsplit(os.sep, 1)[0], "Scripts", "icoextract"])
    else:
        return os.sep.join([sys.executable.rsplit(os.sep, 1)[0], "icoextract"])


# def icoextract(icon_path, output_path, num=None, verbose=False):
#     """Windows and Linux only, uses tis-waptpython-icoextract

#     usage: icoextract [-h] [-V] [-n NUM] [-v] input output

#     Windows PE EXE icon extractor.

#     positional arguments:
#         input              input filename
#         output             output filename

#     optional arguments:
#         -h, --help         show this help message and exit
#         -V, --version      show program's version number and exit
#         -n NUM, --num NUM  index of icon to extract
#         -v, --verbose      enables debug logging

#     """
#     error("no option for 48x48; check https://github.com/jlu5/icoextract/issues/16")
#     try:
#         icoextract_cmd = "".join(
#             [
#                 f'"{get_icoextract_path()}"',
#                 f' "{icon_path}"',
#                 f' "{output_path}"',
#                 f" --num {num}" if num else "",
#                 " --verbose" if verbose else "",
#             ]
#         )
#         print(
#             run(
#                 icoextract_cmd,
#                 accept_returncodes=[0, 1, 3010],
#             )
#         )
#     except:
#         error(f"ERROR: icoextract failed.\nTry to reinstall the package tis-waptpython-icoextract")


def extract_png_from_ico(inputfile, outfile):
    im = Image.open(inputfile)  # Open the .ico from memory

    target_size = (48, 48)
    available_sizes = im.info.get("sizes", [])

    if target_size in available_sizes:
        im.thumbnail(target_size)
        im.save(outfile, "PNG")
        print(f"Successfully extracted {target_size[0]}x{target_size[1]} icon: {inputfile} to: {outfile}")
    else:
        error_message = (
            f"No {target_size[0]}x{target_size[1]} icon found in: {inputfile}\nPlease select another file or improve the code of this function."
        )
        raise ValueError(error_message)


def extract_png_from_bin(inputfile, outfile):
    try:
        extractor = IconExtractor(inputfile)
    except RuntimeError:
        logger.debug("Failed to extract icon for %s:", inputfile, exc_info=True)
        sys.exit(1)

    data = extractor.get_icon()
    im = Image.open(data)  # Open the .ico from memory

    target_size = (48, 48)
    available_sizes = im.info.get("sizes", [])

    if target_size in available_sizes:
        im.thumbnail(target_size)
        im.save(outfile, "PNG")
        print(f"Successfully extracted {target_size[0]}x{target_size[1]} icon: {inputfile} to: {outfile}")
    else:
        error_message = (
            f"No {target_size[0]}x{target_size[1]} icon found in: {inputfile}\nPlease select another file or improve the code of this function."
        )
        raise ValueError(error_message)


def resize_icon_if_needed(file_path: str) -> bool:
    """Resizes the given image file to 48x48 if its dimensions exceed 48x48.

    Args:
        file_path (str): The path of the image file to resize.

    Returns:
        bool: True if the image was resized, False otherwise.
    """
    with Image.open(file_path) as img:
        width, height = img.size
        if width > 48 or height > 48:
            print(f"Resizing icon size from: {width}x{height} to: 48x48 ")
            img.thumbnail((48, 48), Image.ANTIALIAS)
            img.save(file_path)
            return True
        return False


def get_template_path(folder_to_scan, filetype=".py"):
    template_dict = {}
    template_list = []
    if folder_to_scan.split(os.sep)[-1] == "all":
        folder_to_scan = folder_to_scan.rsplit(os.sep, 1)[0]

    files = glob.glob(os.path.join(folder_to_scan, f"*{filetype}"))
    for file_path in files:
        file = os.path.basename(file_path)
        template_dict[file.split(".")[0]] = file_path
    for items in template_dict.items():
        template_list.append(items[0])
    selected_template = waptguihelper.grid_dialog("Please select the proper file", template_list, waptguihelper.GRT_SELECTED)[0]["unknown"]
    return template_dict[selected_template]


def update_control(control, provided_control_dict: Dict[str, str]) -> None:
    """
    Updates the given control file with the provided control dictionary.

    Args:
        control (str or waptpackage.PackageEntry): The path of control file or his PackageEntry call
    """
    if not isinstance(control, PackageEntry):
        pkg_dir = convert_control_path_to_dir(control)
        control = PackageEntry().load_control_from_wapt(pkg_dir)

    for key, value in provided_control_dict.items():
        if str(control[key]) != str(value):
            print(f"Updating: control.{key} from: {control[key]} to: {value}")
            control[key] = value

    control.save_control_to_wapt()


def is_git_repo(repo_dir):
    try:
        repo_url = str(run(f'git -C "{repo_dir}" remote get-url origin', accept_returncodes=[0, 128])).strip()
    except:
        repo_url = "not a git repository"

    if "not a git repository" in repo_url:
        print("INFO: You are editing a package outside a git repository")
        return False

    if "@" in repo_url:
        print("ERROR: The repo is authenticated")
        return False

    if "fatal: detected dubious ownership in repository" in repo_url:
        git_safe_dir_cmd = repo_url.split("\t")[-1]
        run(git_safe_dir_cmd, accept_returncodes=[0, 128])
        repo_url = str(run(f'git -C "{repo_dir}" remote get-url origin', accept_returncodes=[0, 128])).strip()

    repo_package = repo_url.split("/")[-1].split(".git")[0]
    package_name = get_control_dict(get_individual_packages(".")[0])["package"]

    if repo_package != package_name:
        print("ERROR: The repo and the package do not match")
        return False

    return True


def check_repo(repo_dir):
    try:
        repo_url = str(run(f'git -C "{repo_dir}" remote get-url origin', accept_returncodes=[0, 128])).strip()
    except:
        repo_url = "not a git repository"
    if "not a git repository" in repo_url:
        print("INFO: You are editing a package outside a git repository")
        return "not a git repository"
    if "@" in repo_url:
        print("ERROR: The repo is authenticated")
    if "fatal: detected dubious ownership in repository" in repo_url:
        git_safe_dir_cmd = repo_url.split("\t")[-1]
        run(git_safe_dir_cmd, accept_returncodes=[0, 128])
        repo_url = str(run(f'git -C "{repo_dir}" remote get-url origin', accept_returncodes=[0, 128])).strip()
    repo_package = repo_url.split("/")[-1].split(".git")[0]
    package_name = get_control_dict(get_individual_packages(".")[0])["package"]
    if repo_package != package_name:
        print("ERROR: The repo and the package do not match")
    return "a git repository"


def find_git_folder():
    for current_dir in pathlib.Path().resolve().parents:
        git_folder = current_dir / ".git"
        if git_folder.exists() and git_folder.is_dir():
            return str(current_dir)
    return None


def is_git_repo_tis():
    enterprise_repo_set = {"https://gitea.packages.tranquil.it/wapt-packages/"}
    git_folder = find_git_folder()
    if git_folder is not None:
        repo_url = str(run_notfatal("git config --get remote.origin.url", cwd=git_folder)).strip()
        return any(enterprise_repo in repo_url for enterprise_repo in enterprise_repo_set)
    else:
        return False


def update_repo(repo_dir):
    print(run(f"git -C {repo_dir} pull --rebase --autostash", accept_returncodes=[0, 128]))


def get_individual_packages(root_dir: str) -> List[str]:
    """
    Returns a list of all directories that contain a setup.py file
    in the given root directory and its subdirectories.
    """
    package_dirs: List[str] = []

    # Look for setup.py in subdirectories of root_dir
    setup_files: List[str] = glob.glob(makepath(root_dir, "**", "setup.py"), recursive=True)
    for setup_file in setup_files:
        package_dir: str = os.path.dirname(setup_file)
        if package_dir == makepath(root_dir, "template"):
            continue
        package_dirs.append(package_dir)

    # Look for setup.py in the root directory
    root_setup_path: str = makepath(root_dir, "setup.py")
    if os.path.isfile(root_setup_path):
        package_dirs.append(root_dir)

    return package_dirs


def get_package_dirs(root_dir: str, include_disabled: bool = False) -> List[str]:
    """
    Retrieve a list of directory paths containing setup.py files
    within the specified root directory and its subdirectories.

    Args:
        root_dir (str): The root directory to start searching from.
        include_disabled (bool, optional): Whether to include directories
            with disabled setups (file names containing 'setup' and 'disable').
            Defaults to False.

    Returns:
        List[str]: A list of directory paths containing setup.py files.
    """
    package_dirs: List[str] = []

    # Look for setup.py in root_dir
    setup_files: List[str] = glob.glob(os.path.join(root_dir, "**", "setup.py"), recursive=True)
    if include_disabled:
        disabled_setup_files = glob.glob(os.path.join(root_dir, "**", "setup*disable*"), recursive=True)
        setup_files += disabled_setup_files

    for setup_file in setup_files:
        package_dir: str = os.path.dirname(setup_file)
        package_dirs.append(package_dir)

    return package_dirs


def clean_control_dict(
    control_dict: dict,
    values_to_clean=[
        "package_uuid",
        "icon_sha256sum",
        "signer",
        "signer_fingerprint",
        "signature",
        "signature_date",
        "signed_attributes",
    ],
) -> dict:
    """
    Set the values of specified keys to an empty string in a dictionary.

    Args:
        control_dict: The dictionary to be cleaned.
        values_to_clean: A list of keys whose values should be set to an empty string.

    Returns:
        A new dictionary with the specified values set to an empty string.
    """
    if not isinstance(control_dict, dict):
        raise ValueError("Input must be a dictionary.")

    for key in values_to_clean:
        if key in control_dict and key != "":
            control_dict[key] = ""

    return control_dict


def update_values(new_control_dict: dict, control_dict: dict, values_to_update: List[str]) -> dict:
    """
    Update control values in control_dict with values from new_control_dict for a specified list of keys.

    Args:
        new_control_dict: A dictionary containing new control values to update.
        control_dict: A dictionary containing existing control values to be updated.
        values_to_update: A list of keys for the control values to update.

    Returns:
        A dictionary containing the updated control values.
    """
    if not isinstance(new_control_dict, dict) or not isinstance(control_dict, dict):
        raise ValueError("Both input arguments must be dictionaries.")

    for key in values_to_update:
        if control_dict[key] != new_control_dict[key]:
            print(f"Updating {key} from {control_dict[key]} to {new_control_dict[key]}")
            control_dict[key] = new_control_dict[key]

    return control_dict


def get_public_persistent_dir():
    """
    Get the path to the public persistent directory.

    Returns:
        str: Path to the public persistent directory.
    """
    try:
        return makepath(WAPT.wapt_base_dir, "public", "persistent")
    except Exception:
        if get_os_name() == "Windows":
            try:
                return makepath(installed_softwares(uninstallkey="WAPT_is1")[0]["install_location"], "public", "persistent")
            except Exception:
                return makepath(programfiles32, "wapt", "public", "persistent")
        else:
            return "/opt/wapt/public/persistent"


def get_package_tools_path():
    path = makepath(get_public_persistent_dir(), "package-tools")
    return path


def add_enterprise_headers(repo_dir):
    licence_header = """# -*- coding: utf-8 -*-
##################################################
# This file is part of WAPT Enterprise
# All right reserved, (c) Tranquil IT Systems 2024
# For more information please refer to
# https://wapt.tranquil.it/store/licences.html
##################################################
from setuphelpers import *
"""

    for py in glob.glob(makepath(repo_dir, "**", "setup.py"), recursive=True) + glob.glob(
        makepath(repo_dir, "**", "update_package.py"), recursive=True
    ):
        new_lines = [line + "\n" for line in licence_header.splitlines()]

        with open(py, "r", encoding="utf8") as f:
            import_detected = False
            for line in f.readlines():
                if import_detected:
                    new_lines.append(line)
                else:
                    if "from setuphelpers import *" in line:
                        import_detected = True
        if len([i for i in new_lines if "# https://wapt.tranquil.it/store/licences.html" in i]) == 1:
            with open(py, "w", encoding="utf8", newline="\n") as f:
                f.writelines(new_lines)


def remove_accents(input_string):
    """Remove accents from a given string."""
    nfkd_form = unicodedata.normalize("NFKD", input_string)
    return "".join([c for c in nfkd_form if not unicodedata.combining(c)])


def ensure_list_elems_are_unique(csv_or_list, ignore_empty_args=True, allow_none=False):
    """Ensure that list elements are unique.

    Args:
        csv_or_list (list or str): Input list or CSV string to be converted to a list.
        ignore_empty_args (bool): If True, empty string found in csv are not appended to the list.
        allow_none (bool): If True, if csv_or_list is None, return None; else return an empty list.

    Returns:
        list or None
    """
    # Use the ensure_list function to convert input to a list
    input_list = ensure_list(csv_or_list, ignore_empty_args, allow_none)

    # If input_list is None (allow_none=True) or is an empty list, return it as it is
    if input_list is None or len(input_list) == 0:
        return input_list

    # Create a new list to store unique elements while preserving the order
    unique_list = []
    for item in input_list:
        if item not in unique_list:
            unique_list.append(item)

    return unique_list


def make_control_list(input_list, base_list=[], ignore_empty_args=True, allow_none=False):
    """Create a control list with unique elements considering accented characters and starting with base_list.

    Args:
        input_list (list or str): Input list or CSV string to be converted to a list.
        base_list (list): Define the base and beggining of the list.
        ignore_empty_args (bool): If True, empty string found in csv are not appended to the list.
        allow_none (bool): If True, if input_list is None, return None; else return an empty list.

    Returns:
        str of list elements separated by a comma
    """
    input_list = ensure_list_elems_are_unique(input_list, ignore_empty_args, allow_none)
    base_list = ensure_list_elems_are_unique(base_list, ignore_empty_args, allow_none)
    cleaned_list = ensure_list_elems_are_unique(base_list + input_list, ignore_empty_args, allow_none)

    # Remove accents from elements and create a mapping to original elements
    accents_mapping = {}
    cleaned_list_unique = []
    for elem in cleaned_list:
        cleaned_elem = remove_accents(str(elem))
        if cleaned_elem not in accents_mapping:
            accents_mapping[cleaned_elem] = elem
            cleaned_list_unique.append(cleaned_elem)

    # Filter elements from cleaned_list_unique that exist in base_list
    filtered_cleaned_list = [elem for elem in cleaned_list_unique if elem in base_list]

    # Sort the filtered elements based on base_list index
    filtered_cleaned_list.sort(key=base_list.index)

    # Add the remaining elements from cleaned_list_unique that were not in base_list
    remaining_cleaned_list = [elem for elem in cleaned_list_unique if elem not in base_list]
    filtered_cleaned_list.extend(remaining_cleaned_list)

    # Map back to original elements and return the control list
    control_list = [accents_mapping[elem] for elem in filtered_cleaned_list]
    return ",".join(control_list)
