#!waptpython
# -*- coding: utf-8 -*-
import os
import sys

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)))
from setupdevhelpers import *
from dev_utils import *

sys.path.insert(0, sys.executable.rsplit(os.path.sep, 1)[0].replace("/bin", "").replace("\\Scripts", ""))
from setuphelpers import *
import waptguihelper


############################################################################################################
control_file = makepath(".", "WAPT", "control")
provided_control_dict = {}
new_template_control = {}
program_dir = None

# Checking control.target_os (must be lowercase)
update_control(control_file, {"target_os": get_control_dict(control_file)["target_os"].lower()})

# control.name
control_name = get_control_dict(control_file)["name"]
if not control_name:
    complete_control_name(control_file, control_name)

# control.categories
complete_control_categories(control_file)

# *************************************** maintainer ***************************************
maintainer = get_maintainer_name()
if maintainer is None or get_current_user().lower() == maintainer.lower():
    maintainer = waptguihelper.input_dialog("package_template.py", "Who is in charge of the maintenance of this package (for example, John Doe):", "")
    save_maintainer_name(maintainer)
maintainers = ensure_list_elems_are_unique(get_control_dict(control_file)["maintainer"])
maintainers.append(get_maintainer_name())
if get_current_user().lower() != maintainer.lower():
    [maintainers.remove(m) for m in maintainers if m.lower() == get_current_user().lower()]

provided_control_dict.update({"maintainer": make_control_list(maintainers, "WAPT Team,Tranquil IT")})
update_control(control_file, provided_control_dict)


# Checking control.description
if not get_control_dict(control_file).get("description"):
    complete_control_description(control_file)


# *************************************** min_wapt_version ***************************************
complete_control_min_wapt_version(control_file)

# Checking control.installed_size
installed_size = get_control_dict(control_file)["installed_size"]
if installed_size:
    if "\u202f" in str(installed_size):
        installed_size = installed_size.replace("\u202f", "").strip()
        provided_control_dict.update({"installed_size": installed_size})
        update_control(control_file, provided_control_dict)
if installed_size == "" or get_control_dict(control_file)["impacted_process"] == "":
    program_dir = waptguihelper.directory_dialog("Please select the installed location of the software (do not specify if not relevant)")

if installed_size == "" and program_dir is not None:
    installed_size = get_size(program_dir)
    provided_control_dict.update({"installed_size": installed_size})
    update_control(control_file, provided_control_dict)

# *************************************** impacted_process ***************************************
impacted_process_dict = complete_control_impacted_process(control_file, program_dir)
impacted_process = get_control_dict(control_file).get("impacted_process", "")
if impacted_process:
    update_control(control_file, {"impacted_process": ensure_list_elems_are_unique(impacted_process)})


# *************************************** licence ***************************************
# Get the licence from the control file
licences = ensure_list(get_control_dict(control_file).get("licence", ""))

# Check if the licence is valid
VALID_LICENCES = [
    "opensource_free",
    "proprietary_free",
    "proprietary_restricted",
]
if not any(l in licences for l in VALID_LICENCES):
    # If the licence is not valid, prompt the user to select a valid one
    selected_licence_type = ask_grid("Select a licence type", VALID_LICENCES, result_type="GRT_SELECTED", raise_error=False)
    if selected_licence_type:
        licences = []
        licences.append(selected_licence_type[0]["unknown"])


# Check OPEN_SOURCE_LICENCES
OPEN_SOURCE_LICENCES = [
    "cpe:/a:apache:license_2.0",
    "cpe:/a:gnu:gpl_v2",
    "cpe:/a:gnu:gpl_v3",
    "cpe:/a:gnu:agpl_v3",
    "cpe:/a:mozilla:public_license_2.0",
    "cpe:/a:mit:x11_license",
    "cpe:/a:bsd:bsd_license",
    "cpe:/a:eclipse:epl_2.0",
    "cpe:/a:gnu:lgpl_v2.1",
    "cpe:/a:gnu:lgpl_v3",
]
if "opensource_free" in licences and len(licences) < 3 and not any(l in licences for l in OPEN_SOURCE_LICENCES):
    # If the user selects the opensource_free licence type, prompt them to select a valid opensource licence
    additional_licences = ask_grid(
        "Select one or more open-source licence",
        OPEN_SOURCE_LICENCES,
        result_type="GRT_SELECTED",
    )
    if additional_licences:
        [licences.append(l["unknown"]) for l in additional_licences]

# Update the licence in the control file
provided_control_dict.update({"licence": make_control_list(licences)})
update_control(control_file, provided_control_dict)


# Check WAPT_LICENCES
WAPT_LICENCES = [
    "wapt_enterprise",  #   Store WAPT Enterprise package
    "wapt_public",  #       Store WAPT Public package
    "wapt_private",  #      External WAPT package
]
check_repo(".")  # Print if package is not gitted
if is_git_repo_tis():
    if "wapt_private" in licences:
        licences.remove("wapt_private")
else:
    if not "wapt_private" in licences:
        licences.append("wapt_private")
    print("INFO: This package is considered as an External WAPT Package")

# Avoid issue when wapt_enterprise is misspelled
if "wapt_entreprise" in licences:
    licences.remove("wapt_entreprise")
    licences.append("wapt_enterprise")

if not any(l in licences for l in WAPT_LICENCES):
    selected_wapt_licence_type = ask_grid("Select a WAPT licence type", WAPT_LICENCES, result_type="GRT_SELECTED", raise_error=False)
    if selected_wapt_licence_type:
        if len(selected_wapt_licence_type) == 1:
            licences.append(selected_wapt_licence_type[0]["unknown"])
        else:
            print("INFO: Only one WAPT licence can be specified.")

if "wapt_enterprise" in licences:
    add_enterprise_headers(".")
    print("INFO: This package is considered as a Store WAPT Enterprise package")
if "wapt_public" in licences:
    print("INFO: This package is considered as a Store WAPT Public package")

# Update the licence in the control file
provided_control_dict.update({"licence": make_control_list(licences)})
update_control(control_file, provided_control_dict)

# *************************************** os_version ***************************************
# os_version must be 2 digits at least
min_os_version = get_control_dict(control_file)["min_os_version"]
max_os_version = get_control_dict(control_file)["max_os_version"]
min_os_version = min_os_version + ".0" if min_os_version and len(min_os_version.split(".")) == 1 else min_os_version
max_os_version = max_os_version + ".0" if max_os_version and len(max_os_version.split(".")) == 1 else max_os_version

provided_control_dict.update({"min_os_version": min_os_version, "max_os_version": max_os_version})
update_control(control_file, provided_control_dict)


# *************************************** icon.png ***************************************
icon_path = icon_makepath(".")

if isfile(icon_path):
    print("icon.png already exists")
else:
    if program_dir and impacted_process_dict and len(impacted_process_dict) == 1 and get_os_name() == "Windows":
        icon_file = list(impacted_process_dict.values())[0]
        icon_file = ask_filename(
            "Please accept preselected or provide a compatible file to get your package icon",
            os.path.dirname(icon_file),
            icon_file,
            "Extractable Icon Files|*.png;*.exe;*.dll;*.ico",
        )
    else:
        icon_file = ask_filename(
            "Please provide a compatible file to get your package icon",
            program_dir if program_dir else "",
            "",
            "Extractable Icon Files|*.png;*.exe;*.dll;*.ico",
        )
    if icon_file:
        if icon_file.endswith(".png"):
            filecopyto(icon_file, icon_path)
        elif icon_file.endswith(".ico"):
            # Uses tis-waptpython-pillow
            extract_png_from_ico(icon_file, icon_path)
        elif not get_os_name() == "Darwin":
            # # Uses tis-extracticon
            # extract_icon(icon_file, icon_path)
            # Uses tis-waptpython-icoextract
            extract_png_from_bin(icon_file, icon_path)
        else:
            error("ERROR: macOS cannot extract PNG from Windows PE files.")
    else:
        standard_icon_path = makepath(get_package_tools_path(), "wapt-generic-iconset")
        new_icon_file = ask_filename("Please select a PNG for your package", standard_icon_path, "", "PNG Files|*.png", True)
        filecopyto(new_icon_file, icon_path)

# Uses tis-waptpython-pillow
resize_icon_if_needed(icon_path)


# *************************************** changelog.txt ***************************************
changelog_txt_path = changelog_txt_makepath(".")


# *************************************** precommit ***************************************
# cleaning up the control
provided_control_dict = clean_control_dict(get_control_dict(control_file))
update_control(control_file, provided_control_dict)

complete_control_descriptions(control_file, silent=True)

# Formatting code with black
formatting_print = str(run_notfatal("waptpython -m black -l 150 ."))
if "reformatted" in formatting_print:
    print(formatting_print.split("\r\nAll done!")[0].split("\nAll done!")[0])
