# -*- coding: utf-8 -*-
from setuphelpers import *


def install():
    package_tools_path = makepath(get_public_persistent_dir(), "package-tools")
    vscode_launch_custom_path = makepath(WAPT.wapt_base_dir, "templates", "vscode_launch_custom.json")

    if not isdir(get_public_persistent_dir()):
        print(f'Creating: "{get_public_persistent_dir()}"')
        mkdirs(get_public_persistent_dir())

    if isdir(package_tools_path):
        remove_tree(package_tools_path)

    print(f'Copying: "{package_tools_path}"')
    copytree2(makepath(basedir, "package-tools"), package_tools_path)

    print(f'Updating: "{vscode_launch_custom_path}"')
    update_vscode_launches(
        vscode_launch_custom_path,
        package_tools_path,
    )
    filecopyto(vscode_launch_custom_path, makepath(WAPT.wapt_base_dir, "templates", "vscode_launch.json"))  # Until WAPT code is adapted

    # Installing tis-waptpython-icoextract for Windows and Linux
    if not get_os_name() == "Darwin":
        dependency_package = control.package.split("-")[0] + "-waptpython-icoextract"
        if WAPT.is_available(dependency_package):
            WAPT.install(dependency_package)
        else:
            error(f"You must import the package: {dependency_package}")


def audit():
    package_tools_path = makepath(get_public_persistent_dir(), "package-tools")
    vscode_launch_custom_path = makepath(WAPT.wapt_base_dir, "templates", "vscode_launch_custom.json")
    audit_status = "OK"

    if not isdir(package_tools_path):
        print("Missing: %s\nReinstalling package..." % package_tools_path)
        WAPT.install(control.package, force=True)
        audit_status = "WARNING"
    if not isfile(vscode_launch_custom_path):
        print("Missing: %s\nReinstalling package..." % package_tools_path)
        WAPT.install(control.package, force=True)
        audit_status = "WARNING"
    return audit_status


def uninstall():
    package_tools_path = makepath(get_public_persistent_dir(), "package-tools")

    if isdir(package_tools_path):
        print("Removing: %s" % package_tools_path)
        remove_tree(package_tools_path)
    uninstall_vscode_launches()


def update_vscode_launches(vscode_launch_custom_path, package_tools_path):
    new_launches_json = makepath(package_tools_path, "package-template", "vscode_launch_custom.json")
    # launch_path = makepath(WAPT.wapt_base_dir, "templates", "vscode_launch.json")
    launch_path = makepath(package_tools_path, "package-template", "vscode_launch.json")
    launch_content = json_load_file(makepath(launch_path))

    # getting every property including "DEV"
    configuration_to_remove = []
    for i in range(len(launch_content["configurations"])):
        if "DEV" in launch_content["configurations"][i]["name"]:
            configuration_to_remove.append(i)

    # removing every index in reverse order
    for i in reversed(configuration_to_remove):
        # print("removing %s" % launch_content["configurations"][i]["name"])
        del launch_content["configurations"][i]

    new_launches = json_load_file(new_launches_json)
    existing_launches = [launch["name"] for launch in launch_content["configurations"]]

    for launch in new_launches["configurations"]:
        if launch["name"] not in existing_launches:
            # print(f"adding {launch['name']} to vscode_launch.json")
            launch["program"] = launch["program"].replace(
                r"{package_tools_path}/", "/".join(package_tools_path.split(os.sep) + ["package-template", ""])
            )
            launch_content["configurations"].append(launch)

    # Write the modified data back to the file
    # print("updating vscode launchs.json")
    json_write_file(vscode_launch_custom_path, launch_content)


def uninstall_vscode_launches():
    launch_path = makepath(WAPT.wapt_base_dir, "templates", "vscode_launch.json")
    launch_content = json_load_file(launch_path)

    # getting every property including "DEV"
    configuration_to_remove = []
    for i in range(len(launch_content["configurations"])):
        if "DEV" in launch_content["configurations"][i]["name"]:
            configuration_to_remove.append(i)

    # removing every index in reverse order
    for i in reversed(configuration_to_remove):
        # print("removing %s" % launch_content["configurations"][i]["name"])
        del launch_content["configurations"][i]
    # Write the clean data back to the file
    json_write_file(launch_path, launch_content)


def get_public_persistent_dir():
    """
    Get the path to the public persistent directory.

    Returns:
        str: Path to the public persistent directory.
    """
    try:
        return makepath(WAPT.wapt_base_dir, "public", "persistent")
    except Exception:
        if get_os_name() == "Windows":
            try:
                return makepath(installed_softwares(uninstallkey="WAPT_is1")[0]["install_location"], "public", "persistent")
            except Exception:
                return makepath(programfiles32, "wapt", "public", "persistent")
        else:
            return "/opt/wapt/public/persistent"
